#!/bin/bash

# v 1.0 (after some previous unnumbered versions) [12-09-2009]

get_data() {
  OPF=$(sed -n -e '/<rootfile /s/.*full-path="\([^"]*\)".*/\1/p' META-INF/container.xml)
  BASEDIR=$(dirname $OPF)
  dos2unix "$OPF"
  TITLE=$(sed -n -e '/<dc:title/s/.*<dc:title>\(.*\)<\/dc:title>/\1/p' "$OPF")
  NCX=$(sed -n -e '/<spine/s/.*toc="\(.[^"]*\)".*/\1/p' "$OPF")
  NCX=$(sed -n -e "/id=\"$NCX\"/s/.*href=\"\(.[^\"]*\)\".*/\1/p" "$OPF")
  NCX=$BASEDIR/$NCX
  dos2unix "$NCX"
}

write_css() {
cat > indexAAA.css << EOF
body {
  margin: auto auto;
  padding: 2%;
  max-width: 35em;
  text-align: justify;
  font-size: 12pt;
  font-family: serif;
  background-color: white;
}

div.pagebreak {
  border: dashed;
  border-width: 1px 0 0 0;
  margin: 0 -2.05% 0 -2.05%;
}
EOF
}

write_html() {
cat > indexAAA.html << EOF
<html>
<head>
  <title>$TITLE</title>
</head>
<body>
  <a href="$BASEDIR/${files[0]}">$TITLE</a><br>
EOF

for (( i=1; i<${#files[*]}; i++ )); do
  if [[ ${linears[$i]} -eq 1 ]]; then 
      echo "<a href=\"$BASEDIR/${files[$i]}\">${ids[$i]}</a><br/>" >> indexAAA.html
  fi
done

cat >> indexAAA.html << EOF
</body>
</html>

EOF
}

write_xhtml() {
cat > indexAAA.xhtml << EOF
</head>
<body>

EOF

echo -n "'${files[0]}'" >> indexAAA.xhtml
for (( i=1; i<${#files[*]}; i++ )); do
  if [[ ${linears[$i]} -eq 1 ]]; then echo -n ", '${files[$i]}'" >> indexAAA.xhtml; fi
done
echo '];' >> indexAAA.xhtml

cat >> indexAAA.xhtml << EOF
<div class="top">
<table><tr>
<td><a href="${files[0]}" target="book" onclick="setLinks(0);">&lt;&lt;&lt;</a></td>
<td><a class="deact" id="prev" href="${files[0]}" target="book" onclick="prevFile(); return false;">&lt;</a></td>
<td><a id="next" href="${files[1]}" target="book" onclick="nextFile(); return false;">&gt;</a></td>
<td><a href="${files[${#files[*]}-1]}" target="book" onclick="setLinks(files.length-1);">&gt;&gt;&gt;</a></td>
</tr><tr>
<td id="current" class="big" colspan="2">TOC</td>
<td id="change" class="small" onclick="setSpine();">SPINE</td>
</tr></table>
</div>

<div id="spine" class="hide">
EOF

for (( i=0; i<${#ids[*]}; i++ )); do
  j=i
  echo "<a href=\"${files[$i]}\" target=\"book\" onclick=\"setLinks($j);\">${ids[$i]}</a><br/>" >> indexAAA.xhtml
  index[${#index[*]}]=$j
done

cat >> indexAAA.xhtml << EOF
</div>

<div id="toc" class="show">
EOF

for (( i=0; i<${#items[*]}; i++ )); do
  k=-1
  check=$(expr ${srcs[$i]} : '\([^#]*\)#\?.*$')
  for (( j=0; j<${#files[*]}; j++ )); do
    if [[ "$check" == "${files[$j]}" ]]; then k=$j; fi
  done
  echo "<p class=\"level${levels[$i]}\">&bull; <a href=\"${srcs[$i]}\" target=\"book\" onclick=\"setLinks(${index[$k]});\">${items[$i]}</a></p>" >> indexAAA.xhtml
done

cat >> indexAAA.xhtml << EOF
</div>

</body>
</html>
EOF
}

read_spine() {
  ORIGIFS=$IFS
  IFS=`echo -en "\n\b"`
  for id in $(sed -n -e '/<itemref/s/.*idref="\(.[^"]*\)".*/\1/p' "$OPF"); do
    file=$(sed -n -e "/id=\"$id\"/s/.*href=\"\(.[^\"]*\)\".*/\1/p" "$OPF")
    linear=$(sed -n -e "/idref=\"$id\"/s/.*linear=\"\([^\"]*\)\".*/\1/p" "$OPF")
    if [[ $linear == "no" ]]; then linear=0; else linear=1; fi
    ids[${#ids[*]}]=$id
    files[${#files[*]}]=$file
    linears[${#linears[*]}]=$linear
  done
  IFS=$ORIGFS
}

read_toc() {
  ORIGIFS=$IFS
  IFS=`echo -en "\n\b"`
  level=0
  for line in $(cat "$NCX"); do
    if (( $(expr $line : '\s*<navPoint') )); then
      level=$(($level+1))
    elif (( $(expr $line : '\s*<\/navPoint') )); then
      level=$(($level-1))
    elif (( $(expr $line : '\s*<text') )); then
      if (( $level > 0 )); then
        item=$(echo $line | sed -n -e 's/.*<text>\(.*\)<\/text>.*/\1/p')
        items[${#items[*]}]=$item
        levels[${#levels[*]}]=$level
      fi
    elif (( $(expr $line : '\s*<content') )); then
      if (( $level > 0 )); then
        src=$(echo $line | sed -n -e 's/.*src="\(.[^"]*\)".*/\1/p')
        srcs[${#srcs[*]}]=$src
      fi
    fi
  done
  IFS=$ORIGFS
}

#=================================================

while getopts ":o" OPTION; do
  case "$OPTION" in
    o ) OPEN=1
  esac
done
shift $(($OPTIND-1))

DIR=$PWD
EPUB=$(readlink -f "$1")
TEMPDIR=$(mktemp -td epub.XXXXXX) || exit 1
[ -d $TEMPDIR ] || mkdir $TEMPDIR

cd $TEMPDIR
unzip -qo "$EPUB"

get_data
read_spine
read_toc
write_css
write_html
write_xhtml

echo -e "\n$(readlink -f indexAAA.html)\n"

if (( $OPEN )); then
#  kfmclient exec file:$(readlink -f indexAAA.html)
  glinks file:$(readlink -f indexAAA.html)
fi

read -p "Press <ENTER> to delete the temporary files" END

cd $DIR
rm -r $TEMPDIR
exit
